//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

#ifndef _LCDC
#define _LCDC

// HEADERS ==========================================================
#include "lcd.h"
// END HEADERS ======================================================

// C FILES ==========================================================
// END C FILES ======================================================

// MACROS =========================================================
// END MACROS =====================================================

// GLOBALS ==========================================================
struct lcd_map {              // This structure is overlayed
           int  data : 4;
		   bool nu5;
		   bool nu6;
		   bool nu7;
		   bool nu8;
        } lcd;

#byte lcd    = 0xF80

BYTE lcdline;

BYTE const LCD_INIT_STRING[4] = {0x20 | (lcd_type << 2), LCD_OFF, LCD_CLEAR, LCD_ENTRY_MODE_SET};
// These bytes need to be sent to the LCD to start it up.

// END GLOBALS ======================================================

//----------------------------------------------------------------------------------
void lcd_printf(BYTE x, BYTE y, char * str)
//----------------------------------------------------------------------------------
{
   lcd_gotoxy(x, y);
   printf(lcd_putc, str);
}

//----------------------------------------------------------------------------------
void lcd_screen_on()
//----------------------------------------------------------------------------------
{
    set_tris_write();

    output_low(LCD_RS);
    output_low(LCD_RW);
	 output_low(LCD_E);
    delay_ms(15);

    lcd_send_byte(0, LCD_CURSOR_OFF);  //Turn the screen on

    //Turn on the power and let it initialise itself
    output_high(LCD_POWER);
}

//----------------------------------------------------------------------------------
void lcd_screen_off()
//----------------------------------------------------------------------------------
{
    set_tris_write();

    //Turn off the power to the LED backlight
	 output_low(LCD_POWER);

    output_low(LCD_RS);
    output_low(LCD_RW);
	output_low(LCD_E);
    delay_ms(15);

    lcd_send_byte(0, LCD_OFF);  //Turn the screen off
}

//----------------------------------------------------------------------------------
void lcd_init()
//----------------------------------------------------------------------------------
{
    BYTE i;
    set_tris_write();

    output_low(LCD_RS);
    output_low(LCD_RW);
    output_low(LCD_E);
    delay_ms(15);

    for(i = 1; i <= 3; ++i)
	{
       lcd_send_nibble(LCD_INIT_NIBBLE);
       delay_ms(5);
    }

	lcd_send_nibble(LCD_FOUR_BIT);

	for(i = 0; i <= 3; ++i)
	{
       lcd_send_byte(0, LCD_INIT_STRING[i]);
	}
}

//----------------------------------------------------------------------------------
void set_tris_read()
//----------------------------------------------------------------------------------
{
	  set_tris_A(TRISA_READ);
      // For read mode data pins are input
}

//----------------------------------------------------------------------------------
void set_tris_write()
//----------------------------------------------------------------------------------
{
	  set_tris_A(TRISA_WRITE);
      // For write mode all pins are output
}

//----------------------------------------------------------------------------------
BYTE lcd_read_byte()
//----------------------------------------------------------------------------------
{
      BYTE low, high;
      set_tris_read();
      output_high(LCD_RW);
      delay_cycles(1);
	  output_high(LCD_E);
      delay_cycles(1);
      high = lcd.data;
      output_low(LCD_E);
      delay_cycles(1);
	  output_high(LCD_E);
      delay_us(1);
      low = lcd.data;
      output_low(LCD_E);
      set_tris_write();
      return ((high<<4) | low);
}

//----------------------------------------------------------------------------------
void lcd_send_nibble(BYTE n)
//----------------------------------------------------------------------------------
{
      lcd.data = n;
      delay_cycles(1);
	  output_high(LCD_E);
      delay_us(2);
	  output_low(LCD_E);
}

//----------------------------------------------------------------------------------
void lcd_send_byte(bool address_bit, BYTE n)
//----------------------------------------------------------------------------------
{
      output_low(LCD_RS);

      while (bit_test(lcd_read_byte(),7));

	   if (address_bit == 0)
		  output_low(LCD_RS);
	   else
		  output_high(LCD_RS);

      delay_cycles(1);
      output_low(LCD_RW);
      delay_cycles(1);
	  output_low(LCD_E);
      lcd_send_nibble(n >> 4);
      lcd_send_nibble(n & 0x0f);
}

//----------------------------------------------------------------------------------
void lcd_gotoxy(BYTE x, BYTE y)
//----------------------------------------------------------------------------------
{
	BYTE address;

	switch (y)
	{
		case 1: address = lcd_line_one; break;
		case 2: address = lcd_line_two; break;
		case 3: address = lcd_line_three; break;
		case 4: address = lcd_line_four; break;
        default: address = 0; break;
	}

	address += x - 1;
	lcd_send_byte(0, address);
}

//----------------------------------------------------------------------------------
void lcd_putc(char c)
//----------------------------------------------------------------------------------
{
   switch (c)
   {
     case '\f'   : lcd_send_byte(0, 1);
                   lcdline = 1;
                   delay_ms(2);
				   break;
     case '\n'   : lcd_gotoxy(1, ++lcdline);break;
     case '\b'   : lcd_send_byte(0, 0x10);  break;
     default     : lcd_send_byte(1, c);     break;
   }
}

//----------------------------------------------------------------------------------
char lcd_getc(BYTE x, BYTE y)
//----------------------------------------------------------------------------------
{
   char value;

   lcd_gotoxy(x,y);
   output_high(LCD_RS);
   value = lcd_read_byte();
   output_low(LCD_RS);
   return (value);
}

//----------------------------------------------------------------------------------
void lcd_clear_line(BYTE line)
//----------------------------------------------------------------------------------
{
	lcd_gotoxy (0, line);
	printf (lcd_putc, "                    ");
	lcd_gotoxy (0, line);
}

// =============================================================
#endif
