//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

// HEADERS ==========================================================
#include "main.h"
// END HEADERS ======================================================

// C FILES ==========================================================
#include "global.c"
// END C FILES ======================================================

// MACROS =========================================================
// END MACROS =====================================================

// GLOBALS ==========================================================
// END GLOBALS ======================================================

//----------------------------------------------------------------------------------
void initialise()
//----------------------------------------------------------------------------------
{
	rs232_errors = 0;
	set_tris_A(TRISA_READ);
	set_tris_B(TRISB);
	set_tris_C(TRISC);
	output_low(CONVST);
	port_b_pullups(true);
	reset();

	disable_interrupts(global);
	setup_spi(FALSE);
	setup_timer_0(RTCC_INTERNAL);
	setup_timer_1(T1_DISABLED);
	setup_timer_2(T2_DISABLED, 0, 1);
	setup_comparator(NC_NC_NC_NC);
	setup_oscillator(False);
	adc_init();
	lcd_init();
	lcd_screen_off();

	initialise_timer();
	run_timer();
}

//----------------------------------------------------------------------------------
void read_setup()
//----------------------------------------------------------------------------------
{
	//Now read cal info from EEPROM
	load_calibration_info();

	//Setup everything else
	pressure[0] = 0;
	pressure[1] = 0;
	sample_num[0] = 0;
	sample_num[1] = 0;
	dual_channel = !input(CHANNELS);
	load_test_standard_info();
}

//----------------------------------------------------------------------------------
void reset()
//----------------------------------------------------------------------------------
{
	testing = false;
	calibrating = false;
	test_state[0] = state_idle;
	test_state[1] = state_idle;
	reset_solenoids(0);
	reset_solenoids(1);
}

//----------------------------------------------------------------------------------
void switch_on()
//----------------------------------------------------------------------------------
{
	int8 c = 0;

	reset();
	read_setup();

	unit_on = true;
	lcd_screen_on();

	load_string_name(lcdStr);
	lcd_printf(1, 1, lcdStr);

	load_string_version_hw(lcdStr);
	lcd_printf(1, 2, lcdStr);

	load_string_version_sw(lcdStr);
	lcd_printf(1, 3, lcdStr);

	load_string_serial(lcdStr, serial_no);
	lcd_printf(1, 4, lcdStr);

	while ((c < SPLASHTIME) && (!input(ONOFFKEY)))
	{
		c++;
		delay_ms(100);
	}

	if (c == SPLASHTIME)
	{
		update_test_status();
	}
}

//----------------------------------------------------------------------------------
void switch_off()
//----------------------------------------------------------------------------------
{
	reset();
	lcd_screen_off();
	unit_on = false;
}

//----------------------------------------------------------------------------------
bool check_start_key()
//----------------------------------------------------------------------------------
{
	bool pressed;
	
	pressed = false;

	if (!input(STARTKEY))
	{
		if (allow_start_key_press)
		{
			start_key_press++;
			if (start_key_press >= STARTPRESSTIME)
			{
				pressed = true;
				allow_start_key_press = false;
				if (!calibrating)
					start_testing();
				start_key_press = 0;
			}
		}
	}
	else
	{
		start_key_press = 0;
		allow_start_key_press = !testing;
	}

	return pressed;
}

//----------------------------------------------------------------------------------
void main()
//----------------------------------------------------------------------------------
{
	unsigned int8 command;

	initialise();

	#ignore_warnings 203 //Condition always true
	while (true)
	{
		if (unit_on)
		{
			if (timer_has_ticked())
			{
				timer_restart();
				check_start_key();
				update_test_status();
			}

			command = check_command();

   			switch (command)
   			{
   				case CMD_NOOP:
   					break;
   				default:
   					interpret_command(command);
   					break;
   			}
		}

		if (!input(ONOFFKEY))
		{
			if (!unit_on)
			{
				switch_on();
			}
		}
		else
		{
			if (unit_on)
			{
				switch_off();
			}
		}

	}
	#ignore_warnings NONE
}

// =============================================================
